@@ -1,4 +1,4 @@
-# Copyright (C) 2017 Open Information Security Foundation
+# Copyright (C) 2017-2022 Open Information Security Foundation
 # Copyright (c) 2015-2017 Jason Ish
 #
 # You can copy, redistribute or modify this Program under the terms of
@@ -88,7 +88,7 @@ else:
     logger = logging.getLogger()
 
 # If Suricata is not found, default to this version.
-DEFAULT_SURICATA_VERSION = "4.0.0"
+DEFAULT_SURICATA_VERSION = "6.0.0"
 
 # The default filename to use for the output rule file. This is a
 # single file concatenating all input rule files together.
@@ -99,19 +99,26 @@ INDEX_EXPIRATION_TIME = 60 * 60 * 24 * 14
 # Rule keywords that come with files
 file_kw = ["filemd5", "filesha1", "filesha256", "dataset"]
 
+def strict_error(msg):
+    logger.error(msg)
+    if config.args().fail:
+        sys.exit(1)
+
 class Fetch:
 
     def __init__(self):
         self.istty = os.isatty(sys.stdout.fileno())
 
-    def check_checksum(self, tmp_filename, url):
+    def check_checksum(self, tmp_filename, url, checksum_url=None):
         try:
-            checksum_url = url + ".md5"
+            if not isinstance(checksum_url, str):
+                checksum_url = url[0] + ".md5"
+            net_arg=(checksum_url,url[1])
             local_checksum = hashlib.md5(
                 open(tmp_filename, "rb").read()).hexdigest().strip()
             remote_checksum_buf = io.BytesIO()
             logger.info("Checking %s." % (checksum_url))
-            net.get(checksum_url, remote_checksum_buf)
+            net.get(net_arg, remote_checksum_buf)
             remote_checksum = remote_checksum_buf.getvalue().decode().strip()
             logger.debug("Local checksum=|%s|; remote checksum=|%s|" % (
                 local_checksum, remote_checksum))
@@ -163,7 +170,7 @@ class Fetch:
             logger.info("Using latest cached version of rule file: %s", url)
             if not os.path.exists(tmp_filename):
                 logger.error("Can't proceed offline, "
-                             "source %s has not yet been downloaded.", url)
+                             "source {} has not yet been downloaded.".format(url))
                 sys.exit(1)
             return self.extract_files(tmp_filename)
         if not config.args().force and os.path.exists(tmp_filename):
@@ -174,7 +181,7 @@ class Fetch:
                     url)
                 return self.extract_files(tmp_filename)
             if checksum:
-                if self.check_checksum(tmp_filename, url):
+                if self.check_checksum(tmp_filename, net_arg, checksum):
                     logger.info("Remote checksum has not changed. "
                                 "Not fetching.")
                     return self.extract_files(tmp_filename)
@@ -191,14 +198,15 @@ class Fetch:
             tmp_fileobj.close()
         except URLError as err:
             if os.path.exists(tmp_filename):
-                logger.warning(
-                    "Failed to fetch %s, "
-                    "will use latest cached version: %s", url, err)
+                if config.args().fail:
+                    strict_error("Failed to fetch {}: {}".format(url, err))
+                else:
+                    logger.error("Failed to fetch {}, will use latest cached version: {}".format(url, err))
                 return self.extract_files(tmp_filename)
             raise err
         except IOError as err:
             self.progress_hook_finish()
-            logger.error("Failed to copy file: %s", err)
+            logger.error("Failed to copy file: {}".format(err))
             sys.exit(1)
         except Exception as err:
             raise err
@@ -214,7 +222,7 @@ class Fetch:
                 files.update(fetched)
             except URLError as err:
                 url = url[0] if isinstance(url, tuple) else url
-                logger.error("Failed to fetch %s: %s", url, err)
+                strict_error("Failed to fetch {}: {}".format(url, err))
         else:
             for url in self.args.url:
                 files.update(self.fetch(url))
@@ -227,6 +235,8 @@ class Fetch:
 
         # The file is not an archive, treat it as an individual file.
         basename = os.path.basename(filename).split("-", 1)[1]
+        if not basename.endswith(".rules"):
+            basename = "{}.rules".format(basename)
         files = {}
         files[basename] = open(filename, "rb").read()
         return files
@@ -240,12 +250,16 @@ def load_filters(filename):
             line = line.strip()
             if not line or line.startswith("#"):
                 continue
-            line = line.rsplit(" #")[0]
+            line = line.rsplit(" #")[0].strip()
 
-            line = re.sub(r'\\\$', '$', line)  # needed to escape $ in pp
             try:
-                rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
-                filters.append(rule_filter)
+                if line.startswith("metadata-add"):
+                    rule_filter = matchers_mod.AddMetadataFilter.parse(line)
+                    filters.append(rule_filter)
+                else:
+                    line = re.sub(r'\\\$', '$', line)  # needed to escape $ in pp
+                    rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
+                    filters.append(rule_filter)
             except Exception as err:
                 raise exceptions.ApplicationError(
                     "Failed to parse modify filter: {}".format(line))
@@ -295,6 +309,7 @@ def load_local(local, files):
         if len(local_files) == 0:
             local_files.append(local)
         for filename in local_files:
+            filename = os.path.realpath(filename)
             logger.info("Loading local file %s" % (filename))
             if filename in files:
                 logger.warn(
@@ -304,7 +319,7 @@ def load_local(local, files):
                 with open(filename, "rb") as fileobj:
                     files.append(SourceFile(filename, fileobj.read()))
             except Exception as err:
-                logger.error("Failed to open %s: %s" % (filename, err))
+                logger.error("Failed to open {}: {}".format(filename, err))
 
 def load_dist_rules(files):
     """Load the rule files provided by the Suricata distribution."""
@@ -359,7 +374,7 @@ def load_dist_rules(files):
                 with open(path, "rb") as fileobj:
                     files.append(SourceFile(path, fileobj.read()))
             except Exception as err:
-                logger.error("Failed to open %s: %s" % (path, err))
+                logger.error("Failed to open {}: {}".format(path, err))
                 sys.exit(1)
 
 def load_classification(suriconf, files):
@@ -422,24 +437,65 @@ def manage_classification(suriconf, files):
 def handle_dataset_files(rule, dep_files):
     if not rule.enabled:
         return
-    load_attr = [el.strip() for el in rule.dataset.split(",") if "load" in el][0]
-    dataset_fname = os.path.basename(load_attr.split(" ")[1])
-    filename = [fname for fname, content in dep_files.items() if fname == dataset_fname]
-    if filename:
-        logger.debug("Copying dataset file %s to output directory" % dataset_fname)
-        with open(os.path.join(config.get_output_dir(), dataset_fname), "w+") as fp:
-            fp.write(dep_files[dataset_fname].decode("utf-8"))
+    dataset_load = [el for el in (el.strip() for el in rule.dataset.split(",")) if el.startswith("load")]
+    if not dataset_load:
+        # No dataset load found.
+        return
+    dataset_filename = dataset_load[0].split(maxsplit=1)[1].strip()
+
+    # Get the directory name the rule is from.
+    prefix = os.path.dirname(rule.group)
+
+    # Construct the source filename.
+    source_filename = os.path.join(prefix, dataset_filename)
+
+    # If a source filename starts with a "/", look for it on the filesystem. The archive
+    # unpackers will take care of removing a leading / so this shouldn't happen for
+    # downloaded rulesets.
+    if source_filename.startswith("/"):
+        if not os.path.exists(source_filename):
+            logger.warn("Local dataset file '{}' was not found for rule {}, rule will be disabled".format(source_filename, rule.idstr))
+            rule.enabled = False
+            return
+        dataset_contents = open(source_filename, "rb").read()
     else:
-        logger.error("Dataset file %s was not found" % dataset_fname)
+        if not source_filename in dep_files:
+            logger.warn("Dataset file '{}' was not found for rule {}, rule will be disabled".format(dataset_filename, rule.idstr))
+            rule.enabled = False
+            return
+        dataset_contents = dep_files[source_filename]
+
+    source_filename_hash = hashlib.md5(source_filename.encode()).hexdigest()
+    new_rule = re.sub(r"(dataset.*?load\s+){}".format(dataset_filename), r"\g<1>datasets/{}".format(source_filename_hash), rule.format())
+    dest_filename = os.path.join(config.get_output_dir(), "datasets", source_filename_hash)
+    dest_dir = os.path.dirname(dest_filename)
+    logger.debug("Copying dataset file {} to {}".format(dataset_filename, dest_filename))
+    try:
+        os.makedirs(dest_dir, exist_ok=True)
+    except Exception as err:
+        logger.error("Failed to create directory {}: {}".format(dest_dir, err))
+        return
+    with open(dest_filename, "w") as fp:
+        fp.write(dataset_contents.decode("utf-8"))
+    return new_rule
 
 def handle_filehash_files(rule, dep_files, fhash):
     if not rule.enabled:
         return
     filehash_fname = rule.get(fhash)
-    filename = [fname for fname, content in dep_files.items() if os.path.join(*(fname.split(os.path.sep)[1:])) == filehash_fname]
-    if filename:
+
+    # Get the directory name the rule is from.
+    prefix = os.path.dirname(rule.group)
+
+    source_filename = os.path.join(prefix, filehash_fname)
+    dest_filename = source_filename[len(prefix) + len(os.path.sep):]
+    logger.debug("dest_filename={}".format(dest_filename))
+
+    if source_filename not in dep_files:
+        logger.error("{} file {} was not found".format(fhash, filehash_fname))
+    else:
         logger.debug("Copying %s file %s to output directory" % (fhash, filehash_fname))
-        filepath = os.path.join(config.get_state_dir(), os.path.dirname(filename[0]))
+        filepath = os.path.join(config.get_output_dir(), os.path.dirname(dest_filename))
         logger.debug("filepath: %s" % filepath)
         try:
             os.makedirs(filepath)
@@ -447,11 +503,10 @@ def handle_filehash_files(rule, dep_files, fhash):
             if oserr.errno != errno.EEXIST:
                 logger.error(oserr)
                 sys.exit(1)
-        logger.debug("output fname: %s" % os.path.join(filepath, os.path.basename(filehash_fname)))
-        with open(os.path.join(filepath, os.path.basename(filehash_fname)), "w+") as fp:
-            fp.write(dep_files[os.path.join("rules", filehash_fname)].decode("utf-8"))
-    else:
-        logger.error("%s file %s was not found" % (fhash, filehash_fname))
+        output_filename = os.path.join(filepath, os.path.basename(filehash_fname))
+        logger.debug("output fname: %s" % output_filename)
+        with open(output_filename, "w") as fp:
+            fp.write(dep_files[source_filename].decode("utf-8"))
 
 def write_merged(filename, rulemap, dep_files):
 
@@ -489,13 +544,17 @@ def write_merged(filename, rulemap, dep_files):
     with io.open(tmp_filename, encoding="utf-8", mode="w") as fileobj:
         for sid in rulemap:
             rule = rulemap[sid]
+            reformatted = None
             for kw in file_kw:
                 if kw in rule:
                     if "dataset" == kw:
-                        handle_dataset_files(rule, dep_files)
+                        reformatted = handle_dataset_files(rule, dep_files)
                     else:
                         handle_filehash_files(rule, dep_files, kw)
-            print(rule.format(), file=fileobj)
+            if reformatted:
+                print(reformatted, file=fileobj)
+            else:
+                print(rule.format(), file=fileobj)
     os.rename(tmp_filename, filename)
 
 def write_to_directory(directory, files, rulemap, dep_files):
@@ -508,9 +567,9 @@ def write_to_directory(directory, files, rulemap, dep_files):
 
     oldset = {}
     if not args.quiet:
-        for filename in files:
+        for file in files:
             outpath = os.path.join(
-                directory, os.path.basename(filename))
+                directory, os.path.basename(file.filename))
 
             if os.path.exists(outpath):
                 for rule in rule_mod.parse_file(outpath):
@@ -533,26 +592,30 @@ def write_to_directory(directory, files, rulemap, dep_files):
                         len(removed),
                         len(modified)))
 
-    for filename in sorted(files):
+    for file in sorted(files):
         outpath = os.path.join(
-            directory, os.path.basename(filename))
+            directory, os.path.basename(file.filename))
         logger.debug("Writing %s." % outpath)
-        if not filename.endswith(".rules"):
-            open(outpath, "wb").write(files[filename])
+        if not file.filename.endswith(".rules"):
+            open(outpath, "wb").write(file.content)
         else:
             content = []
-            for line in io.StringIO(files[filename].decode("utf-8")):
+            for line in io.StringIO(file.content.decode("utf-8")):
                 rule = rule_mod.parse(line)
-                if not rule:
+                if not rule or rule.id not in rulemap:
                     content.append(line.strip())
                 else:
+                    reformatted = None
                     for kw in file_kw:
                         if kw in rule:
                             if "dataset" == kw:
-                                handle_dataset_files(rule, dep_files)
+                                reformatted = handle_dataset_files(rulemap[rule.id], dep_files)
                             else:
-                                handle_filehash_files(rule, dep_files, kw)
-                    content.append(rulemap[rule.id].format())
+                                handle_filehash_files(rulemap[rule.id], dep_files, kw)
+                    if reformatted:
+                        content.append(reformatted)
+                    else:
+                        content.append(rulemap[rule.id].format())
             tmp_filename = ".".join([outpath, "tmp"])
             io.open(tmp_filename, encoding="utf-8", mode="w").write(
                 u"\n".join(content))
@@ -619,13 +682,15 @@ def dump_sample_configs():
 def resolve_flowbits(rulemap, disabled_rules):
     flowbit_resolver = rule_mod.FlowbitResolver()
     flowbit_enabled = set()
+    pass_ = 1
     while True:
+        logger.debug("Checking flowbits for pass %d of rules.", pass_)
         flowbits = flowbit_resolver.get_required_flowbits(rulemap)
         logger.debug("Found %d required flowbits.", len(flowbits))
         required_rules = flowbit_resolver.get_required_rules(rulemap, flowbits)
         logger.debug(
-            "Found %d rules to enable to for flowbit requirements",
-            len(required_rules))
+            "Found %d rules to enable for flowbit requirements (pass %d)",
+            len(required_rules), pass_)
         if not required_rules:
             logger.debug("All required rules enabled.")
             break
@@ -637,15 +702,16 @@ def resolve_flowbits(rulemap, disabled_rules):
             rule.enabled = True
             rule.noalert = True
             flowbit_enabled.add(rule)
+        pass_ = pass_ + 1
     logger.info("Enabled %d rules for flowbit dependencies." % (
         len(flowbit_enabled)))
 
 class ThresholdProcessor:
 
     patterns = [
-        re.compile("\s+(re:\"(.*)\")"),
-        re.compile("\s+(re:(.*?)),.*"),
-        re.compile("\s+(re:(.*))"),
+        re.compile(r"\s+(re:\"(.*)\")"),
+        re.compile(r"\s+(re:(.*?)),.*"),
+        re.compile(r"\s+(re:(.*))"),
     ]
 
     def extract_regex(self, buf):
@@ -885,6 +951,9 @@ def load_sources(suricata_version):
                         "Source index is required for source %s; "
                         "run suricata-update update-sources" % (source["source"]))
                 source_config = index.get_source_by_name(name)
+                if source_config is None:
+                    logger.warn("Source no longer exists in index and will not be fetched: {}".format(name))
+                    continue
                 try:
                     checksum = source_config["checksum"]
                 except:
@@ -924,9 +993,14 @@ def load_sources(suricata_version):
     # Now download each URL.
     files = []
     for url in urls:
+
+        # To de-duplicate filenames, add a prefix that is a hash of the URL.
+        prefix = hashlib.md5(url[0].encode()).hexdigest()
         source_files = Fetch().run(url)
         for key in source_files:
-            files.append(SourceFile(key, source_files[key]))
+            content = source_files[key]
+            key = os.path.join(prefix, key)
+            files.append(SourceFile(key, content))
 
     # Now load local rules.
     if config.get("local") is not None:
@@ -1017,7 +1091,7 @@ def _main():
     ]
     for arg in unimplemented_args:
         if hasattr(args, arg) and getattr(args, arg):
-            logger.error("--%s not implemented", arg)
+            logger.error("--{} not implemented".format(arg))
             return 1
 
     suricata_path = config.get("suricata")
@@ -1028,7 +1102,7 @@ def _main():
         # The Suricata version was passed on the command line, parse it.
         suricata_version = engine.parse_version(args.suricata_version)
         if not suricata_version:
-            logger.error("Failed to parse provided Suricata version: %s" % (
+            logger.error("Failed to parse provided Suricata version: {}".format(
                 args.suricata_version))
             return 1
         logger.info("Forcing Suricata version to %s." % (suricata_version.full))
@@ -1055,7 +1129,7 @@ def _main():
         elif hasattr(args, "func"):
             return args.func()
         elif args.subcommand != "update":
-            logger.error("Unknown command: %s", args.subcommand)
+            logger.error("Unknown command: {}".format(args.subcommand))
             return 1
 
     if args.dump_sample_configs:
@@ -1075,27 +1149,39 @@ def _main():
 
     # Load user provided disable filters.
     disable_conf_filename = config.get("disable-conf")
-    if disable_conf_filename and os.path.exists(disable_conf_filename):
-        logger.info("Loading %s.", disable_conf_filename)
-        disable_matchers += load_matchers(disable_conf_filename)
+    if disable_conf_filename:
+        if os.path.exists(disable_conf_filename):
+            logger.info("Loading %s.", disable_conf_filename)
+            disable_matchers += load_matchers(disable_conf_filename)
+        else:
+            logger.warn("disable-conf file does not exist: {}".format(disable_conf_filename))
 
     # Load user provided enable filters.
     enable_conf_filename = config.get("enable-conf")
-    if enable_conf_filename and os.path.exists(enable_conf_filename):
-        logger.info("Loading %s.", enable_conf_filename)
-        enable_matchers += load_matchers(enable_conf_filename)
+    if enable_conf_filename:
+        if os.path.exists(enable_conf_filename):
+            logger.info("Loading %s.", enable_conf_filename)
+            enable_matchers += load_matchers(enable_conf_filename)
+        else:
+            logger.warn("enable-conf file does not exist: {}".format(enable_conf_filename))
 
     # Load user provided modify filters.
     modify_conf_filename = config.get("modify-conf")
-    if modify_conf_filename and os.path.exists(modify_conf_filename):
-        logger.info("Loading %s.", modify_conf_filename)
-        modify_filters += load_filters(modify_conf_filename)
+    if modify_conf_filename:
+        if os.path.exists(modify_conf_filename):
+            logger.info("Loading %s.", modify_conf_filename)
+            modify_filters += load_filters(modify_conf_filename)
+        else:
+            logger.warn("modify-conf file does not exist: {}".format(modify_conf_filename))
 
     # Load user provided drop filters.
     drop_conf_filename = config.get("drop-conf")
-    if drop_conf_filename and os.path.exists(drop_conf_filename):
-        logger.info("Loading %s.", drop_conf_filename)
-        drop_filters += load_drop_filters(drop_conf_filename)
+    if drop_conf_filename:
+        if os.path.exists(drop_conf_filename):
+            logger.info("Loading %s.", drop_conf_filename)
+            drop_filters += load_drop_filters(drop_conf_filename)
+        else:
+            logger.warn("drop-conf file does not exist: {}".format(drop_conf_filename))
 
     # Load the Suricata configuration if we can.
     suriconf = None
@@ -1112,7 +1198,7 @@ def _main():
     # Disable rule that are for app-layers that are not enabled.
     if suriconf:
         for key in suriconf.keys():
-            m = re.match("app-layer\.protocols\.([^\.]+)\.enabled", key)
+            m = re.match(r"app-layer\.protocols\.([^\.]+)\.enabled", key)
             if m:
                 proto = m.group(1)
                 if not suriconf.is_true(key, ["detection-only"]):
@@ -1171,6 +1257,12 @@ def _main():
 
     for key, rule in rulemap.items():
 
+        # To avoid duplicate counts when a rule has more than one modification
+        # to it, we track the actions here then update the counts at the end.
+        enabled = False
+        modified = False
+        dropped = False
+
         for matcher in disable_matchers:
             if rule.enabled and matcher.match(rule):
                 logger.debug("Disabling: %s" % (rule.brief()))
@@ -1181,25 +1273,32 @@ def _main():
             if not rule.enabled and matcher.match(rule):
                 logger.debug("Enabling: %s" % (rule.brief()))
                 rule.enabled = True
-                enable_count += 1
+                enabled = True
 
         for fltr in drop_filters:
             if fltr.match(rule):
-                rulemap[rule.id] = fltr.run(rule)
-                drop_count += 1
+                rule = fltr.run(rule)
+                dropped = True
 
         for fltr in modify_filters:
             if fltr.match(rule):
-                new_rule = fltr.run(rule)
-                if new_rule:
-                    rulemap[rule.id] = new_rule
-                    modify_count += 1
+                rule = fltr.run(rule)
+                modified = True
+
+        if enabled:
+            enable_count += 1
+        if modified:
+            modify_count += 1
+        if dropped:
+            drop_count += 1
+
+        rulemap[key] = rule
 
     # Check if we should disable ja3 rules.
     try:
         disable_ja3(suriconf, rulemap, disabled_rules)
     except Exception as err:
-        logger.error("Failed to dynamically disable ja3 rules: %s" % (err))
+        logger.error("Failed to dynamically disable ja3 rules: {}".format(err))
 
     # Check rule vars, disabling rules that use unknown vars.
     check_vars(suriconf, rulemap)
@@ -1218,7 +1317,7 @@ def _main():
     # Check that output directory is writable.
     if not os.access(config.get_output_dir(), os.W_OK):
         logger.error(
-            "Output directory is not writable: %s", config.get_output_dir())
+            "Output directory is not writable: {}".format(config.get_output_dir()))
         return 1
 
     # Backup the output directory.
@@ -1234,10 +1333,10 @@ def _main():
         file_tracker.add(output_filename)
         write_merged(os.path.join(output_filename), rulemap, dep_files)
     else:
-        for filename in files:
+        for file in files:
             file_tracker.add(
                 os.path.join(
-                    config.get_output_dir(), os.path.basename(filename)))
+                    config.get_output_dir(), os.path.basename(file.filename)))
         write_to_directory(config.get_output_dir(), files, rulemap, dep_files)
 
     manage_classification(suriconf, classification_files)
@@ -1281,7 +1380,7 @@ def _main():
         logger.info("Running %s." % (config.get("reload-command")))
         rc = subprocess.Popen(config.get("reload-command"), shell=True).wait()
         if rc != 0:
-            logger.error("Reload command exited with error: %d", rc)
+            logger.error("Reload command exited with error: {}".format(rc))
 
     logger.info("Done.")
 
